package com.xclele.bigevent.controller;

import com.xclele.bigevent.pojo.Result;
import com.xclele.bigevent.pojo.User;
import com.xclele.bigevent.service.UserService;
import com.xclele.bigevent.utils.JwtUtil;
import com.xclele.bigevent.utils.Md5Util;
import com.xclele.bigevent.utils.ThreadLocalUtil;
import jakarta.validation.constraints.Pattern;
import jdk.dynalink.Operation;
import org.hibernate.validator.constraints.URL;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/user")
@Validated
public class UserController {

    @Autowired
    private UserService userService;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @PostMapping("/register")
    public Result register(@Pattern(regexp = "^\\S{5,16}$") String username, String password) {
        //@Pattern用来校验参数
        //search for users
        User u = userService.findByUsername(username);
        if (u == null) {
            // No existing
            userService.register(username, password);
            return Result.success();
        } else {
            // Exists
            return Result.error("Username already exists");
        }

    }

    @PostMapping("/login")
    public Result<String> login(@Pattern(regexp = "^\\S{5,16}$") String username, String password) {
        //@Pattern用来校验参数
        //search for users
        User loginUsr = userService.findByUsername(username);
        if (loginUsr == null) {
            // No existing
            return Result.error("Username does not exist");
        }

        // check password
        if (Md5Util.getMD5String(password).equals(loginUsr.getPassword())){
            Map<String,Object> claims = new HashMap<>();
            claims.put("id",loginUsr.getId());
            claims.put("username",loginUsr.getUsername());
            String token = JwtUtil.genToken(claims);
            //把token存到redis
            ValueOperations<String, String> operations = stringRedisTemplate.opsForValue();
            operations.set(token, token, 15, TimeUnit.MINUTES);

            return Result.success(token);
        }

        return Result.error("Password error");

    }

    @GetMapping("/userinfo")
    public Result<User> userInfo(/*@RequestHeader(name="Authorization") String token*/) {
        //根据用户名查询用户
//        Map<String, Object> map = JwtUtil.parseToken(token);
//        String username = (String) map.get("username");
        Map<String,Object> map = ThreadLocalUtil.get();
        String username = (String) map.get("username");

        User user = userService.findByUsername(username);
        return Result.success(user);
    }

    @PutMapping("/update")
    public Result update(@RequestBody @Validated User user){
        userService.update(user);
        return Result.success();
    }

    @PatchMapping("/updateavatar")
    public Result updateAvatar(@RequestParam @URL String avatarUrl){
        userService.updateAvatar(avatarUrl);
        return Result.success();
    }

    @PatchMapping("/updatePwd")
    public Result updatePwd(@RequestBody Map<String,String> params, @RequestHeader("Authorization") String token){
        String oldPwd = params.get("old_pwd");
        String newPwd = params.get("new_pwd");
        String confirmPwd = params.get("re_pwd");

        if (!StringUtils.hasLength(oldPwd) || !StringUtils.hasLength(newPwd) || !StringUtils.hasLength(confirmPwd)){
            return Result.error("Please enter all fields");
        };

        //原密码是否匹配
        Map<String,Object> map = ThreadLocalUtil.get();
        String username = (String) map.get("username");
        User loginUser = userService.findByUsername(username);

        if (!loginUser.getPassword().equals(Md5Util.getMD5String(oldPwd))){
            return Result.error("old Password error");
        };

        //newpwd和confirmPwd是否一致
        if (!newPwd.equals(confirmPwd)){
            return Result.error("new Password and confirm Password are inconsistent");
        };

        //调用service实现更新
        userService.updatePwd(newPwd);
        //删除redis里面旧的token
        ValueOperations<String, String> operations = stringRedisTemplate.opsForValue();
        operations.getOperations().delete(token);

        return Result.success();
    }


}
